data_types = [
    'extension',

    'bool',

    'int8',
    'int16',
    'int32',
    'int64',
    'uint8',
    'uint16',
    'uint32',
    'uint64',
    'bignum',

    'float',
    'complex',

    'bytes',
    'bytearray',

    'string',

    'list',
    'tuple',
    'set',
    'frozenset',
    'dict',

    'range',

    'class',
    'none',
]
